@@ -1,22 +1,22 @@
 import builtins as exceptions
 #many unsupported attribute of types lib in python 3
 import types
-import re, math 
+import re, math
 from collections import defaultdict
 import numpy
 import scipy
 import pandas as pd
 
 import cobra
-#from cobra import Metabolite, Reaction, Gene
+from cobra import Metabolite, Gene
 from cobra import Gene
 from .metabolite import Metabolite
 from .reaction import Reaction
 from cobra.flux_analysis import deletion, moma, phenotype_phase_plane
 from cobra.core.solution import get_solution
-#from cobra.manipulation import modify
+from cobra.manipulation import modify
 
-#from ..analysis import FCA, Pareto, RWFM, MOMA, ROOM, GeometricFBA, MinSolve
+from ..analysis import FCA, Pareto, RWFM, MOMA, ROOM, GeometricFBA, MinSolve
 from ..analysis import Graph, FluxSum, FVA, MinSolve, Scan
 from ..manipulation import Reversible
 from ..classes.flux import flux
@@ -26,7 +26,7 @@ from ..io import Network
 
 
 class model(cobra.Model):
-    def __init__(self, existing_model=None, bounds=float('inf')):
+    def __init__(self, existing_model=None, bounds=1000000):
         self.all_reactions = {}
         self.unusable_reactions = {}
         self.all_mets = {}
@@ -39,7 +39,7 @@ class model(cobra.Model):
             self.quadratic_component = None
             self.bounds = bounds
             self.SetBounds(bounds=bounds)
-        
+
 
     #### MANIPULATING AND WRITING MODELS #########################################
 
@@ -63,7 +63,7 @@ class model(cobra.Model):
                 reac.id += sf
             for met in sf_model.metabolites:
                 met.id += sf
-                if isinstance(met.compartment, str): 
+                if isinstance(met.compartment, str):
                     met.compartment += sf
             sf_model.repair()
             big_model.MergeWithModel(sf_model)
@@ -77,8 +77,8 @@ class model(cobra.Model):
             else:
                 if replace_with_new:
                     self.DelReaction(reac.id)
-                    self.add_reaction(reac)  
-                      
+                    self.add_reaction(reac)
+
 #    def WriteModel(self, filename, model_format=None, excel_format="cobra",
 #                   sbml_level=2, sbml_version=1, fbc=False, ExtReacs=[]):
 #        """ model_format = "sbml" | "excel" | "matlab" | "json" | "cobra" | "cobra_old" | "scrumpy" """
@@ -130,11 +130,11 @@ class model(cobra.Model):
             Network.WriteMetabolitesAttributes(self, filename, attributes)
 
     #### REACTIONS, METABOLITES AND GENE DATA ######################
-    
+
     ######## GETTING REACTIONS #############################################
     def GetReaction(self, reac):
         if not isinstance(reac, Reaction):
-            reac = self.reactions[self.reactions.index(reac)]
+            reac = Reaction(self.reactions[self.reactions.index(reac)])
         return reac
 
     def GetReactions(self, reactions):
@@ -143,6 +143,8 @@ class model(cobra.Model):
     def GetReactionName(self, reac):
         if isinstance(reac, Reaction):
             reac = reac.id
+        else:
+            self.GetReaction(reac).id
         return reac
 
     def GetReactionNames(self, reactions):
@@ -172,7 +174,6 @@ class model(cobra.Model):
                 if len(iso_x) > 1:
                     iso.append(iso_x)
         return iso
-        
 
     ######## PRINTING REACTIONS ######################################################
     def PrintReaction(self, reaction, AsMetNames=False):
@@ -193,7 +194,7 @@ class model(cobra.Model):
     def AddReaction(self, reac, stodic, rev=False, bounds=None, name=None,
                     subsystem=None):
         """ bounds = val | (lb,ub) """
-        reaction = Reaction(reac)
+        reaction = Reaction(id=reac)
         if name != None:
             reaction.name = name
         if subsystem != None:
@@ -227,9 +228,9 @@ class model(cobra.Model):
     def DelReaction(self, reaction, delete_metabolites=False, clean=True):
         reaction = self.GetReaction(reaction)
         if delete_metabolites:
-            for met in reaction.metabolites: 
+            for met in reaction.metabolites:
                 self.DelMetabolite(met,clean=clean)
-        self.remove_reactions([reaction])
+        self.remove_reactions([reaction.id])
 
         if clean:
             if reaction.id in self.all_reactions:
@@ -275,6 +276,19 @@ class model(cobra.Model):
                         rv[reac.id] = bal_dict
         return rv
 
+    def GetImbalancedReactionsOnTargets(self, TargetElements, **kwargs):
+        """Get a list of reactions where targets are imbalanced
+
+        Args:
+            TargetElements ([list]): list of elements to be considered
+        """
+        reactions = [self.GetReaction(s) for s in self.Reactions()]
+        return [r for r in reactions if not r.IsBalancedOnTarget(TargetElements, **kwargs)]
+
+    def GetAllImbalancedReactions(self, **kwargs):
+        """ Get a list of all imbalanced reactions """
+        reactions = [self.GetReaction(s) for s in self.Reactions()]
+        return [r for r in reactions if not r.IsBalanced(**kwargs)]
 
     def CheckReactionBalance(self, reac, IncCharge=True, ExcElements=None):
         reac = self.GetReaction(reac)
@@ -332,6 +346,30 @@ class model(cobra.Model):
                     mets.remove(met)
         return mets
 
+    def GetDuplicatedMetabolites(self):
+        """Get {formula: [names]} dictionary of metabolite names having identical formula"""
+        mets = self.Metabolites()
+        formula = [self.GetMetabolite(s).formula for s in mets]
+
+        d = {}
+        for i in range(len(mets)):
+            if formula[i] in d.keys():
+                d[formula[i]].append(mets[i])
+            else:
+                d[formula[i]] = [mets[i]]
+        return d
+
+    def GetMetaboliteDifferences(self, other):
+        """Get {formula: [names]} dictionary of metabolite names having identical formula"""
+        mets = self.Metabolites()
+        mets_ = other.Metabolites()
+
+        diff = []
+        for m in set(mets).intersection(set(mets_)):
+            if self.GetMetabolite(m).elements != other.GetMetabolite(m).elements:
+                diff.append(m)
+        return diff
+
     ####### ADDING AND REMOVING METABOLITES #############################
     def AddMetabolite(self, met, formula=None, name=None, charge=None, compartment=None):
         if met in self.Metabolites():
@@ -344,7 +382,7 @@ class model(cobra.Model):
     def DelMetabolite(self, met, destructive=False, method='substractive',clean=True):
         """ method = 'subtractive'|'destructive' """
         met = self.GetMetabolite(met)
-        if method == 'substractive': 
+        if method == 'substractive':
             destructive = False
         #if method == 'destructive':
         #    for reac in list(met._reaction):
@@ -380,12 +418,12 @@ class model(cobra.Model):
             r.add_metabolites({met_to:iw[r]}, combine=True)
             r.add_metabolites({met_from:-iw[r]}, combine=True)
 
-    def AddProtonsToMets(self,met_proton_dic,proton,ExcReacs=None):
-            self.AddProtonsToMet(met,proton,met_proton_dic[met],ExcReacs=ExcReacs)
+    #def AddProtonsToMets(self,met_proton_dic,proton,ExcReacs=None):
+    #        self.AddProtonsToMet(met,proton,met_proton_dic[met],ExcReacs=ExcReacs)
 
     def AddProtonsToMet(self,met,proton,n_p,ExcReacs=None):
         """
-            This function adds n_p amount of protons to the reactions met is involved in 
+            This function adds n_p amount of protons to the reactions met is involved in
         """
         proton = self.GetMetabolite(proton)
         reactions = self.InvolvedWith(met,'metabolite')
@@ -437,7 +475,7 @@ class model(cobra.Model):
             if self.GetMetabolite(k).formula is None:
                 no_formula_mets[k] = self.GetMetabolite(k)
                 result.append(k)
-                
+
         self.no_formula_mets = no_formula_mets
         return result
 
@@ -465,9 +503,10 @@ class model(cobra.Model):
             if k in exclude:
                 continue
             del self.no_formula_mets[k]
-            del self.all_mets[k]
+            if k in self.all_mets:
+                del self.all_mets[k]
             self.DelMetabolite(k, destructive=destructive, method=method, clean=clean)
-        """  
+        """
         if with_reactions:
             collate = self.ReactionsWithNoForumlaMetabolites()
             for k in reactions_mets_nf:
@@ -566,14 +605,14 @@ class model(cobra.Model):
     def TruncateCompartment(self, component):
         """ component: reaction | metabolite """
         truncated = []
-        
+
         if component == "reaction":
             component = "Reactions"
         elif component == "metabolite":
             component = "Metabolites"
         else:
             raise Exception("Unknown component: "+component)
-            
+
         for v in getattr(self, component)():
             a = v.split("_")
             if len(a) > 1:
@@ -581,16 +620,16 @@ class model(cobra.Model):
             else:
                 truncated.append(v)
         return truncated
-            
+
 
     #### ASSOCIATIONS BETWEEN ATTRIBUTES #####################################
-    
+
     def InvolvedWith(self, thing, thing_type=None, AsName=False):
         """ thing_type = None | "reaction" | "metabolite" """
         """
-            This functions either 
-                1) takes in a reaction object and returns a dict of {<metabolites involved in it> : <stoiciometry> } 
-            or  2) takes in a metabolite object and returns a dict of {<reactions it is involved in> : stoichiometry} 
+            This functions either
+                1) takes in a reaction object and returns a dict of {<metabolites involved in it> : <stoichiometry> }
+            or  2) takes in a metabolite object and returns a dict of {<reactions it is involved in> : stoichiometry}
         """
         if thing in self.Reactions() or thing in self.reactions:
             if thing_type == None or 'reac' in thing_type:
@@ -615,7 +654,7 @@ class model(cobra.Model):
 
     def DictConversion(self, input_dict=None, reaction_dict=None,metabolite_dict=None):
         #TAKES IN AN OBJECT DICTIONARY AND RETURN A NEW DICTIONARY WITH THE ID AS KEY AND OBJECT AS VALUE
-        
+
         if(input_dict and not reaction_dict and not metabolite_dict):
             if(isinstance(list(input_dict.keys())[0],cobra.Metabolite)):
                 metabolite_dict = input_dict
@@ -805,10 +844,16 @@ class model(cobra.Model):
                     lb, ub = lb
         if lb == None:
             lb = -self.bounds
-        reac.lower_bound = lb
         if ub == None:
             ub = self.bounds
-        reac.upper_bound = ub
+
+        if lb > reac.upper_bound:
+            reac.upper_bound = ub
+            reac.lower_bound = lb
+        else:
+            reac.lower_bound = lb
+            reac.upper_bound = ub
+
 
     def SetFixedFlux(self, fluxdic):
         """ pre: fluxdic = {"R1":1} """
@@ -849,7 +894,7 @@ class model(cobra.Model):
             for reac in reacsdic:
                 reacval = reacsdic[reac]
                 reac = self.GetReaction(reac)
-                reac.add_metabolites({metabolite:reacval}) 
+                reac.add_metabolites({metabolite:reacval})
             self.AddReaction(name+"_sum_reaction", {metabolite:-1},
                              bounds=bounds)
 
@@ -897,7 +942,7 @@ class model(cobra.Model):
         for reac in ratiodic:
             temp_rd[self.GetReaction(reac)] = ratiodic[reac]
         reactions = self.GetReactions(temp_rd.keys())
-        
+
         for reac in reactions[1:]:
             metname = reactions[0].id + "_" + reac.id + "_fixedratio"
             self.AddMetabolite(metname)
@@ -915,6 +960,36 @@ class model(cobra.Model):
         else:
             self.DelMetabolites(fixedratio)
 
+    def GreedyConstraintScan(model, constraints, order=None):
+        """ Greedily add constraints until no additional constraint gives solution
+        constraints: {rxn: (lb, ub)} dictionary
+        """
+        import copy
+        cons_d = copy.deepcopy(constraints)
+        cons_subs = {}
+        stop = False
+        while not stop:
+            keys = list(cons_d.keys())
+            if order == 'random':
+                numpy.random.shuffle(keys)
+            for k in keys:
+                t = cons_d[k]
+
+                cons_before = model.GetConstraint(k)
+                model.SetConstraints({k: t})
+                model.Solve()
+                # print(f"after: {model.GetConstraint(k)}")
+                if model.solution.status == "optimal":
+                    print(f"{k} is added")
+                    cons_subs[k] = t
+                    del cons_d[k]
+                    break
+                else:
+                    model.SetConstraints({k: cons_before})
+            else:
+                stop = True
+        model.Solve()
+        return model, cons_subs, list(set(cons_d) - set(cons_subs))
 
     ###### SETTING OBJECTIVES ############################
     def SetObjDirec(self, direc="Min"):
@@ -969,9 +1044,9 @@ class model(cobra.Model):
             self.quadratic_component = None
 
     def GetObjVal(self):
-        if self.solution != None: 
+        if self.solution != None:
             return self.solution.objective_value
-        else: 
+        else:
             #print("no solution found")
             return None
 
@@ -987,19 +1062,19 @@ class model(cobra.Model):
         return obj
 
     #### SOLVING AND DISPLAYING SOLUTION ##################################
-    
+
     ######## SOLVING #################################
     def Solve(self,PrintStatus=True, raise_error=False):
         sol = self.optimize(objective_sense=self.objective_direction, raise_error=raise_error)
         self.latest_solution = sol
         if PrintStatus:
-            try: 
+            try:
                 print(self.solution.status)
-            except AttributeError: 
+            except AttributeError:
                 print("no solution")
         #print(sol.status)
 
-    def MinFluxSolve(self, PrintStatus=True, PrimObjVal=True, norm="linear", 
+    def MinFluxSolve(self, PrintStatus=True, PrimObjVal=True, norm="linear",
                 weighting='uniform', ExcReacs=[], adjusted=False, tol_step=1e-9,
                 max_tol=1e-6, DisplayMsg=False, cobra=True, subopt=1.0):
         """ norm = "linear" | "euclidean"
@@ -1007,14 +1082,14 @@ class model(cobra.Model):
         MinSolve.MinFluxSolve(self, PrintStatus=PrintStatus,
                               PrimObjVal=PrimObjVal, norm=norm,
                               weighting=weighting, ExcReacs=ExcReacs,
-                              adjusted=adjusted, tol_step=tol_step, 
-                              max_tol=max_tol, DisplayMsg=DisplayMsg, 
+                              adjusted=adjusted, tol_step=tol_step,
+                              max_tol=max_tol, DisplayMsg=DisplayMsg,
                               cobra=cobra, subopt=subopt)
         #return solfluxes
 
 #    def AdjustedMinFluxSolve(self, PrintStatus=True, PrimObjVal=True, weighting='uniform', ExcReacs=[],
 #                             SolverName=None, StartToleranceVal = 0,DisplayMsg=False):
-#        
+#
 #        """ Adjusts the Minflux_objective constraint for feasible solution
 #            StartToleranceVal = starting tolerance value"""
 #        MinSolve.AdjustedMinFluxSolve(self, PrintStatus=PrintStatus,
@@ -1038,16 +1113,16 @@ class model(cobra.Model):
         if self.solution != None:
             if self.solution.status == "optimal" and not math.isnan(self.solution.objective_value):
                 return True
-            else: 
+            else:
                 return False
         else:
             #("no solution found")
             return False
 
     def GetStatusMsg(self):
-        if self.solution != None: 
+        if self.solution != None:
             return self.solution.status
-        else: 
+        else:
             #print("no solution")
             return "no solution"
 
@@ -1062,9 +1137,9 @@ class model(cobra.Model):
                 #sol_object = Reversible.MergeSolution(self.solution)
                 #sol = flux(sol_object.fluxes.to_dict())
                 sol = flux(self.solution.fluxes.to_dict())
-            else: 
+            else:
                 print("no optimal solution")
-                sol = flux({})  
+                sol = flux({})
                 return sol
         else:
             sol = flux(sol)
@@ -1100,7 +1175,7 @@ class model(cobra.Model):
             for reac in list(sol.keys()):
                 if reac not in reacs:
                     del sol[reac]
-        if AsID: 
+        if AsID:
             newsol = {}
             for reac in sol.keys():
                 solval = sol[reac]
@@ -1200,20 +1275,20 @@ class model(cobra.Model):
         rv = FVA.FVA(self, reaclist=reaclist, subopt=subopt,
             IncZeroes=IncZeroes, VaryOnly=VaryOnly, AsMtx=AsMtx, tol=tol,
             PrintStatus=PrintStatus, cobra=cobra, processes=processes,
-            loopless=loopless, pfba_factor=pfba_factor,reset_state=reset_state) 
+            loopless=loopless, pfba_factor=pfba_factor,reset_state=reset_state)
         return rv
 
-    def MinFluxFVA(self, reaclist=None, subopt=1.0, IncZeroes=True, 
-                   VaryOnly=False, AsMtx=False, tol=1e-10, PrintStatus=False, 
+    def MinFluxFVA(self, reaclist=None, subopt=1.0, IncZeroes=True,
+                   VaryOnly=False, AsMtx=False, tol=1e-10, PrintStatus=False,
                    cobra=True, processes=None, weighting='uniform', ExcReacs=[],
                    loopless=False, pfba_factor=1.0, reset_state=True):
         rv = FVA.MinFluxFVA(self, reaclist=reaclist, subopt=subopt,
             IncZeroes=IncZeroes, VaryOnly=VaryOnly, AsMtx=AsMtx, tol=tol,
             PrintStatus=PrintStatus, cobra=cobra, processes=processes,
-            weighting=weighting, ExcReacs=ExcReacs, 
+            weighting=weighting, ExcReacs=ExcReacs,
             loopless=loopless, pfba_factor=pfba_factor, reset_state=reset_state)
         return rv
-            
+
     def AllFluxRange(self, tol=1e-10, processes=None, reset_state=True):
         return FVA.AllFluxRange(self, tol=tol, processes=processes, reset_state=reset_state)
 
@@ -1479,26 +1554,26 @@ class model(cobra.Model):
 
 
     ## MODEL COMPARISON FUNCTIONS ####################################################
-    def CompareModel(self, m2): 
+    def CompareModel(self, m2):
         """
-        params self,m2: two model objects to be compared 
-        
+        params self,m2: two model objects to be compared
+
         returns comparison(dict):
-         comparison["reactions"][1] contains a list of reactions unique to only m1 
-         comparison["reactions"][2] contains a list of reactions unique to only m2 
+         comparison["reactions"][1] contains a list of reactions unique to only m1
+         comparison["reactions"][2] contains a list of reactions unique to only m2
          comparison["reactions"][3] contains a list of reactions in both m1 and m2
-         comparison["metabolites"][1] contains a list of reactions unique to only m1 
-         comparison["metabolites"][2] contains a list of reactions unique to only m2 
+         comparison["metabolites"][1] contains a list of reactions unique to only m1
+         comparison["metabolites"][2] contains a list of reactions unique to only m2
          comparison["metabolites"][3] contains a list of reactions in both m1 and m2
-         comparison["genes"][1] contains a list of reactions unique to only m1 
-         comparison["genes"][2] contains a list of reactions unique to only m2 
+         comparison["genes"][1] contains a list of reactions unique to only m1
+         comparison["genes"][2] contains a list of reactions unique to only m2
          comparison["genes"][3] contains a list of reactions in both m1 and m2
         """
         comparison = {}
         comparison["reactions"] = []
         comparison["metabolites"] = []
         comparison["genes"] = []
-    
+
         comparison["reactions"].append(set(self.Reactions()).difference(set(m2.Reactions())))
         comparison["reactions"].append(set(m2.Reactions()).difference(set(self.Reactions())))
         comparison["reactions"].append(set(self.Reactions()).intersection(set(m2.Reactions())))
@@ -1506,11 +1581,11 @@ class model(cobra.Model):
         comparison["metabolites"].append(set(self.Metabolites()).difference(set(m2.Metabolites())))
         comparison["metabolites"].append(set(m2.Metabolites()).difference(set(self.Metabolites())))
         comparison["metabolites"].append(set(self.Metabolites()).intersection(set(m2.Metabolites())))
-    
+
         comparison["genes"].append(set(self.Genes()).difference(set(m2.Genes())))
         comparison["genes"].append(set(m2.Genes()).difference(set(self.Genes())))
         comparison["genes"].append(set(self.Genes()).intersection(set(m2.Genes())))
-    
-        return comparison 
-    
+
+        return comparison
+
 
