@@ -10,7 +10,7 @@ from .ScrumPy import ReadScrumPyModel, WriteScrumPyModel
 from past.builtins import basestring
 
 def ReadModel(model_file=None, model_format=None, excel_parse="cobra_string",
-          variable_name=None, Print=False, compartment_dic={}, bounds=float('inf'), **kwargs):
+          variable_name=None, Print=False, compartment_dic={}, bounds=1000000, **kwargs):
     """ model_format = "sbml" | "sbml_legacy" | "excel" | "matlab" | "json" | "scrumpy | "yaml"
         excel_parse = "cobra_string" | "cobra_position" "cyc" """
     if not model_file:
@@ -127,5 +127,5 @@ def WriteModel(model, filename, model_format=None, excel_format="cobra", ExtReac
             WriteExcel(model_copy, filename, excel_format=excel_format)
         else:
             WriteExcel(model, filename, excel_format=excel_format)
-    else: 
+    else:
         print('Please specify model_format')
